// MIDI Humanizer GUI
// Team La-a
// 20111104

package ladasha.GUI;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.sound.midi.*;
import java.io.*;
import javax.imageio.*;
import java.util.*;
import ladasha.io.*;
import ladasha.statistics.*;
import ladasha.processes.*;

public class GUI {
  // globals, yo
  private static String currentPane = "0"; // first card
  private static int currentPaneIndex = 0; // internal representation
  private static int lastPaneIndex = 6; // last card
  private static JPanel cardpanel; // panel to contain cards
  private static File selectedFile; // the selected file to work on
  private static ArrayList<File> profiles = new ArrayList<File>(); //profiles...
  private static ArrayList<File> stdProfiles = new ArrayList<File>(); //built in profiles
  private static JComboBox<String> profileBox = new JComboBox<String>();
	private static JComboBox<String> trackBox = new JComboBox<String>();
	private static JComboBox<String> parameterBox = new JComboBox<String>();
  private static JButton next; // next button, control panel
  private static JButton prev; // previous button, control panel
  private static JButton help; // help button, control panel
  private static JButton clearModificationsButton = new JButton("Clear");
  private static JLabel bottom; //control panel
  private static JButton assignModificationButton = new JButton("Assign");
  private static JLabel modificationsSelected = new JLabel("<html>\n<center><b> no modifications selected\n</html>");
  private static BufferedImage image;
  private static BufferedImage image2;
  private static BufferedImage image3;
  private static BufferedImage image4;
  private static BufferedImage image5;
  private static MidiFileHandler midifilehandler = MidiFileHandler.getInstance();
  private static HashMap<String,File> modifications = new HashMap<String,File>();
  private static Humanizer humanizer = Humanizer.getInstance();
	private static boolean hasBeenHumanized = false;
  private static JTextField fileName = new JTextField(25);
	private static JProgressBar humanizingBar = new JProgressBar();
	private static JProgressBar time2 = new JProgressBar();
  private static Logger logger = Logger.getInstance();
  private static Sequencer sequencer;
  
  private static void createAndShowGUI() {
    /***************************************************************************
     * Frame Initialization code - Don't touch this except for the size
     **************************************************************************/
    JFrame frame = new JFrame("MIDI Humanizer, by Team La-a");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setIconImage(new ImageIcon("icon.png").getImage());
    frame.setSize(600,450);
   
    /***************************************************************************
     * All the cards - do your work here
     ***************************************************************************
     * Welcome card (index 0)
    ***************************************************************************/
    
    //load images
    try {
      image = ImageIO.read(new File("MIDI.png"));
    } catch (IOException io) {logger.log(io);}
    
    //instantiate and define settings for components
    JLabel card0label = new JLabel(new ImageIcon(image));
    JPanel card0 = new JPanel(new FlowLayout(FlowLayout.CENTER,200,100));
    
    //put everything together
    card0.add(card0label);
    
    /***************************************************************************
     * File selection card (index 1)
     **************************************************************************/

    //load images
    try {
      image4 = ImageIO.read(new File("play.png"));
      image5 = ImageIO.read(new File("stop.png"));
    } catch (IOException io) {logger.log(io);}

		//instantiate and define settings for components
    final JPanel card1 = new JPanel(new BorderLayout());
    final JPanel card1top = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card1center = new JPanel(new FlowLayout(FlowLayout.CENTER,500,20));
    final JPanel card1ctop = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card1ccenter = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card1cbottom = new JPanel(new BorderLayout());
    final JLabel fileLabel = new JLabel("<html><h2>Select a MIDI file to humanize by clicking the button below</h2></html>");
    final JLabel selectedFileLabel = new JLabel("<html><b>no file selected</html>");
    final JButton fileSelectButton = new JButton("Select a file");
    final JButton initialPreviewButton = new JButton(new ImageIcon(image4));
    final JButton stopButton = new JButton(new ImageIcon(image5));
    final JFileChooser fc = new JFileChooser();
    final JProgressBar time = new JProgressBar();
    time.setPreferredSize(new Dimension(200,22));
    fc.setFileFilter(new FileNameExtensionFilter("MIDI Files","midi", "mid"));
    initialPreviewButton.setEnabled(false);
    stopButton.setVisible(false);
     
    //assemble components
    card1.add(card1top,BorderLayout.NORTH);
    card1.add(card1center,BorderLayout.CENTER);
    card1top.add(fileLabel);
    card1ctop.add(fileSelectButton);
    card1ccenter.add(selectedFileLabel);
    card1ccenter.add(initialPreviewButton);
    card1ccenter.add(stopButton);
    card1cbottom.add(time,BorderLayout.NORTH);
    card1center.add(card1ctop);
    card1center.add(card1ccenter);
    card1center.add(card1cbottom);
        
    // add listeners
    fileSelectButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        try{ //show file selection window
          fc.showOpenDialog(card1);
        } catch (HeadlessException ex) {logger.log(ex);}
        selectedFile = fc.getSelectedFile();
        if (selectedFile != null) { //show filename and enable next and previewing
          logger.log("GUI: fileSelectButton: loaded midi file: " + selectedFile.getName());
          selectedFileLabel.setText("<html><b>"
   				+ selectedFile.getName()
					+ "</html>");
	  			midifilehandler.loadMidiFile(selectedFile);
					initialPreviewButton.setEnabled(true);
					time.setStringPainted(true);
          long minutes = midifilehandler.getMidiSequence().getMicrosecondLength()/60000000;
          long seconds = midifilehandler.getMidiSequence().getMicrosecondLength() % 60000000 / 1000000;
          String s;
          if (seconds < 10) s = "0:00/" + minutes + ":" + 0 + seconds;
          else s = "0:00/" + minutes + ":" + seconds;
          time.setString(s);
      		time.setMaximum((int)midifilehandler.getMidiSequence().getMicrosecondLength());
					clearModificationsButton.doClick();	
					next.setEnabled(true);
				}
			}
		});
   
    initialPreviewButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (!midifilehandler.loadMidiFile(selectedFile)) return; // load file, break on failure
        else {
          initialPreviewButton.setVisible(false);
          stopButton.setVisible(true);
					sequencer = midifilehandler.getMidiSequencer();
          final String length = midifilehandler.getMidiSequence().getMicrosecondLength()/60000000 + ":" + 
						(midifilehandler.getMidiSequence().getMicrosecondLength() % 60000000) /1000000;
          midifilehandler.previewUntilStop(); // play preview
          new Thread(new Runnable() {
            public void run() {
            	try {
            		while (!midifilehandler.isPlaying()) {
            	  	Thread.sleep(500);
            	  }
            	  while (midifilehandler.isPlaying()) {
            	    Thread.sleep(100);
            	    long minutes = sequencer.getMicrosecondPosition()/60000000;
            	    long seconds = sequencer.getMicrosecondPosition() % 60000000 / 1000000;
            	    String s;
            	    if (seconds < 10) s = minutes + ":" + 0 + seconds + "/" + length;
            	    else s = minutes + ":" + seconds + "/" + length;
            	    time.setString(s);
            	    time.setValue((int)sequencer.getMicrosecondPosition());
            	  }
									
            	} catch (InterruptedException ie) {}
            		time.setValue(0);
								time.setString("0:00/" + sequencer.getMicrosecondLength()/60000000 +
									":" + sequencer.getMicrosecondLength() % 60000000/ 1000000);
								stopButton.setVisible(false);
            	  initialPreviewButton.setVisible(true);
          	}
        	}).start();
      	}
    	}
  	});

    stopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        // stop the loaded file
        new Thread(new Runnable() {
          public void run() {
            midifilehandler.stop();
            initialPreviewButton.setVisible(true);
            stopButton.setVisible(false);
            try {
              while(time.getValue() > 0) {
                time.setValue(0);
                time.setString("0:00/" + midifilehandler.getMidiSequence().getMicrosecondLength()/60000000 + 
									":" + (midifilehandler.getMidiSequence().getMicrosecondLength() % 60000000) /1000000);
                Thread.sleep(100);
              }
            } catch (InterruptedException ie) {}
          }
        }).start();
      }
    });
 
   
   
    /***************************************************************************
     * Profile selection card (index 2)
     **************************************************************************/
    
    //load images

    //instantiate and define settings for components
    final JPanel card2 = new JPanel(new BorderLayout());
    final JPanel card2top = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card2center = new JPanel(new BorderLayout());
    final JPanel card2ctop = new JPanel(new BorderLayout());
		final JPanel card2ctopt = new JPanel(new BorderLayout());
		final JPanel card2ctoptt = new JPanel(new FlowLayout(FlowLayout.CENTER));
		final JPanel card2ctoptb = new JPanel(new FlowLayout(FlowLayout.CENTER));
		final JPanel card2ccenter = new JPanel(new BorderLayout());
    final JPanel card2cbottom = new JPanel(new FlowLayout(FlowLayout.CENTER));
		final JPanel card2bottom = new JPanel(new FlowLayout());
		final JLabel card2label = new JLabel("<html><h2>Add or create your custom profiles here</h2></html>");
		final JLabel addLabel = new JLabel("<html><h3>Add a saved profile</h3></html>");
    final JLabel loadedProfileList = new JLabel("<html><h3>Custom profiles loaded</h3><p><b>no custom profiles loaded</html>", SwingConstants.CENTER);
    final JButton profileAddButton = new JButton("Add profile...");
 		final JLabel newProfileName = new JLabel("<html><h3>Enter your profile name</h3></html>");
		final JLabel newMean = new JLabel("<html><h3>Mean</h3></html>",SwingConstants.CENTER);
		final JLabel newDeviation = new JLabel("<html><h3>Standard<br>Deviation</h3></html>",SwingConstants.CENTER);
		final JTextField newProfileNameField = new JTextField(20);
		final JSlider newMeanSlider = new JSlider(-100,100);
		final JSlider newDeviationSlider = new JSlider(0,100,0);
		final JButton createProfileButton = new JButton("Create new");
    final JButton clearProfilesButton = new JButton("Clear");
    final JFileChooser pc = new JFileChooser(); 
    final JScrollPane card2cbottomScroll = new JScrollPane(card2cbottom,ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		card2cbottomScroll.setPreferredSize(new Dimension(300,85));
		card2cbottomScroll.setBorder(null);
		newMeanSlider.setPreferredSize(new Dimension(500,50));
		newMeanSlider.setMajorTickSpacing(25);
		newMeanSlider.setMinorTickSpacing(5);
		newMeanSlider.setPaintTicks(true);
		newMeanSlider.setPaintLabels(true);
		newDeviationSlider.setPreferredSize(new Dimension(500,50));
		newDeviationSlider.setMajorTickSpacing(25);
		newDeviationSlider.setMinorTickSpacing(5);
		newDeviationSlider.setPaintTicks(true);
		newDeviationSlider.setPaintLabels(true);
    pc.setFileFilter(new FileNameExtensionFilter("MIDI Humanizer Profiles","mhp", "csv"));
    
    //load built-in profiles
    logger.log("GUI: stdProfiles: loading standard profiles");
    stdProfiles.add(new File("WorldClass.mhp"));
    stdProfiles.add(new File("Pro.mhp"));
    stdProfiles.add(new File("Amateur.mhp"));
    stdProfiles.add(new File("Sophomore.mhp"));
    stdProfiles.add(new File("Newbie.mhp"));
    stdProfiles.add(new File("KidsPlay.mhp"));
    stdProfiles.add(new File("Dis1.mhp"));
    stdProfiles.add(new File("Dis2.mhp"));
    stdProfiles.add(new File("Dis3.mhp"));
    for (File f : stdProfiles)
			profiles.add(f);

    //assemble components
    card2.add(card2top,BorderLayout.NORTH);
    card2.add(card2center,BorderLayout.CENTER);
		card2.add(card2bottom,BorderLayout.SOUTH);
		card2top.add(card2label);
		card2ctopt.add(card2ctoptt,BorderLayout.NORTH);
		card2ctopt.add(card2ctoptb,BorderLayout.SOUTH);
		card2ctoptb.add(newProfileName);
		card2ctoptb.add(newProfileNameField);
    card2center.add(card2ctop,BorderLayout.NORTH);
		card2center.add(card2ccenter,BorderLayout.CENTER);
    card2center.add(card2cbottomScroll,BorderLayout.SOUTH);
		card2ctoptt.add(addLabel);
    card2ctoptt.add(profileAddButton);
    card2ctoptb.add(createProfileButton);
		card2bottom.add(clearProfilesButton);
    card2cbottom.add(loadedProfileList);
		card2ctop.add(card2ctopt,BorderLayout.NORTH);
		card2ctop.add(newMean,BorderLayout.CENTER);
		card2ctop.add(newMeanSlider,BorderLayout.EAST);
		card2ccenter.add(newDeviation,BorderLayout.CENTER);
		card2ccenter.add(newDeviationSlider,BorderLayout.EAST); 
    
    //add listeners
    profileAddButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
				File temp = null;
				int index = 0;
        try{ //show file selection window
          pc.showOpenDialog(card2);
        } catch (HeadlessException ex) {logger.log(ex);}
        temp = pc.getSelectedFile();
        if (temp != null && !profiles.contains(temp)) {
	  		logger.log("GUI: profileAddButton: adding profile " + temp.getName());	
                String fileName = temp.getName();
          profiles.add(temp); //add to profiles list
	  			profileBox.addItem(fileName); //add to card3's profile selection box
          //show loaded profiles
	  			StringBuilder newlabelcontents = new StringBuilder();
          newlabelcontents.append("<html><center><h3>Custom Profiles Loaded</h3>");
          for (File f : profiles) {
	    			if (!stdProfiles.contains(f))
              newlabelcontents.append("<p>" + f.getName());
	  			}
          newlabelcontents.append("</center></html>");
          loadedProfileList.setText(newlabelcontents.toString());
        }
      }
    });
    
  	createProfileButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String s = "[" + newMeanSlider.getValue() + "," + newDeviationSlider.getValue() + "]";
				logger.log("GUI: createProfileButton: making custom gaussian profile -> " + s);
        File temp = new File(newProfileNameField.getText() + ".mhp");
				pc.setSelectedFile(temp);
				int retval = JFileChooser.ERROR_OPTION;
				try {
						retval = pc.showSaveDialog(card2);
				} catch (HeadlessException ex){logger.log(ex);}
				if (retval == JFileChooser.APPROVE_OPTION) {
					FileWriter outputStream = null;
					logger.log("GUI: createProfileButton: saving custom profile to " + pc.getSelectedFile().getName());
        	try{
						outputStream = new FileWriter(pc.getSelectedFile());
						BufferedWriter out = new BufferedWriter(outputStream);
						out.write(s);
						out.close();
					} catch (IOException ex) { logger.log(ex);}
					if (pc.getSelectedFile() != null && !profiles.contains(pc.getSelectedFile())) {
	  				String fileName = pc.getSelectedFile().getName();
        	  profiles.add(pc.getSelectedFile()); //add to profiles list
	  				profileBox.addItem(fileName); //add to card3's profile selection box
        	  //show loaded profiles
	  				StringBuilder newlabelcontents = new StringBuilder();
        	  newlabelcontents.append("<html><center><h3>Custom Profiles Loaded</h3>");
        	  for (File f : profiles) {
	    				if (!stdProfiles.contains(f))
        	      newlabelcontents.append("<p>" + f.getName());
	  				}
        	  newlabelcontents.append("</center></html>");
        	  loadedProfileList.setText(newlabelcontents.toString());
        	}
					JOptionPane.showMessageDialog(createProfileButton,"Profile Saved.");
				}
			}
		});
					 
		clearProfilesButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        logger.log("GUI: clearProfilesButton: clearing loaded custom profiles");
        for (File f : profiles)
	  			if (!stdProfiles.contains(f)) {
	    			profileBox.removeItem(f.getName());
						if (modifications != null) {
								String key = "";
								while (modifications.containsValue(f)) {
									for (String s : modifications.keySet())
										if (f == modifications.get(s)) {
											key = s;
											break;
										}
									modifications.remove(key);
								}
						}
					}
				if (!modificationsSelected.getText().contains("no modifications selected")) {
					logger.log("GUI: clearProfilesButton: clearing custom profiles from modifications selected");
					StringBuilder newlabelcontents = new StringBuilder();
					newlabelcontents.append("<html>\n<table cellpadding =\"2\" cellspacing =\"2\">\n");
        	if (modifications.isEmpty()) {
        	 	newlabelcontents.append("</table><br><center><b>no modifications selected\n");
	      	  next.setEnabled(false);
        	} else {
	      	  next.setEnabled(true);
        	  for (String s : modifications.keySet()) {
	      	    String key = s;
	      	    if (key.length() == 2)
        	      newlabelcontents.append("<tr><td width = \"30\"><center>" + key.substring(0,1) + "</td>");
	      	    else newlabelcontents.append("<tr><td width = \"30\"><center>" + key.substring(0,2) + "</td>");
        	  	if (key.endsWith("V")) newlabelcontents.append("<td width = \"100\"><center>Velocity</td>"); 
        	  	if (key.endsWith("T")) newlabelcontents.append("<td width = \"100\"><center>Timing</td>");
        	  	if (key.endsWith("A")) newlabelcontents.append("<td width = \"100\"><center>Amplitude </td>");
        	  	newlabelcontents.append("<td width = \"150\"><center>" + modifications.get(key).getName() + "</td></tr>\n");
        	  }
						newlabelcontents.append("</html>");
      			modificationsSelected.setText(newlabelcontents.toString());
					}
				}
				profiles.clear();
				next.setEnabled(true);
				for (File f : stdProfiles)
					profiles.add(f);
        loadedProfileList.setText("<html><b>no custom profiles loaded</html>");
			}
    });
   

    
    /***************************************************************************
     * Parameter selection card (index 3)
     **************************************************************************/

    //load images

    
    //instantiate and define settings for components
    final JPanel card3 = new JPanel(new BorderLayout());
    final JPanel card33 = new JPanel(new GridLayout(1,2));
    final JPanel card3top = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card3right = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card3rightCenter = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card3left = new JPanel(new FlowLayout(FlowLayout.CENTER,0,25));
    final JPanel card3leftTop = new JPanel(new FlowLayout(FlowLayout.CENTER,0,50));
    final JPanel card3leftTop1 = new JPanel(new BorderLayout());
    final JPanel card3leftTop2 = new JPanel(new BorderLayout());
    final JPanel card3leftTop3 = new JPanel(new BorderLayout());
    final JPanel card3leftBottom = new JPanel(new FlowLayout(FlowLayout.CENTER,10,0));
    final JLabel card3label = new JLabel("<html><h2>Select Parameters to Modify</h2></html>");
    final JLabel modificationLabel = new JLabel("<html>\n<center><h3>Modifications selected:</h3>\n<table cellpadding =\"2\" cellspacing =\"2\">\n<tr>" + 
		"<td width = \"30\"><center><b>Track</td><td width = \"100\"><center><b>Parameter</td><td width = 150><center><b>Profile</td></tr></table></html>");
    final JScrollPane card3rightScroll = new JScrollPane(card3rightCenter,ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
    card3leftTop.setPreferredSize(new Dimension(275,225));
    card3leftTop1.setPreferredSize(new Dimension(275,22));
    card3leftTop2.setPreferredSize(new Dimension(275,22));
    card3leftTop3.setPreferredSize(new Dimension(275,22));
    card3rightScroll.setBorder(null);
    card3rightScroll.setPreferredSize(new Dimension(300,250));
    assignModificationButton.updateUI();
		clearModificationsButton.updateUI();


    //selectable tracks
    trackBox.setPreferredSize(new Dimension(175,25));
    trackBox.addItem("All");
		trackBox.updateUI();
    for (int i = 1; i <= 16; i++)
      trackBox.addItem("Channel " + i);
    final JLabel trackLabel = new JLabel("<html><b>Channel:  </b></html>",SwingConstants.RIGHT);
    
    //selectable parameters
    final JLabel parameterLabel = new JLabel("<html><b>Parameter:  </b></html>",SwingConstants.RIGHT);
    String[] parameters = new String[4];
    parameters[0] = "All";
    parameters[1] ="Velocity";
    parameters[2] = "Timing";
    parameters[3] = "Amplitude";
		for (int i = 0; i <=3; i++)
	    parameterBox.addItem(parameters[i]);
    parameterBox.setPreferredSize(new Dimension(175,25));
    parameterBox.updateUI();

    //selectable profiles, starting with built in mhp's
    final JLabel profileLabel = new JLabel("<html><b>Profile:  </b></html>",SwingConstants.RIGHT);
    profileBox.setPreferredSize(new Dimension(175,25));
    profileBox.updateUI();
		profileBox.addItem("none");
    for (File f : stdProfiles)
      profileBox.addItem(f.getName());
    
    //assemble components
    card3.add(card3top,BorderLayout.NORTH);
    card3.add(card33,BorderLayout.CENTER);
    card33.add(card3left);
    card33.add(card3right);
    card3right.add(modificationLabel);
    card3right.add(card3rightScroll);
    card3rightCenter.add(modificationsSelected);
    card3top.add(card3label);
    card3left.add(card3leftTop);
    card3left.add(card3leftBottom);
    card3leftTop.add(card3leftTop1);
    card3leftTop.add(card3leftTop2);
    card3leftTop.add(card3leftTop3);
    card3leftTop1.add(trackLabel,BorderLayout.CENTER);
    card3leftTop1.add(trackBox,BorderLayout.EAST);
    card3leftTop2.add(parameterLabel,BorderLayout.CENTER);
    card3leftTop2.add(parameterBox,BorderLayout.EAST);
    card3leftTop3.add(profileLabel,BorderLayout.CENTER);
    card3leftTop3.add(profileBox,BorderLayout.EAST);
    card3leftBottom.add(assignModificationButton);
    card3leftBottom.add(clearModificationsButton);
    
    //add listeners
    assignModificationButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        StringBuilder newlabelcontents = new StringBuilder();
	      String selectedProfile = profileBox.getSelectedItem().toString();
	      String selectedTrack = trackBox.getSelectedItem().toString();
	      String selectedParameter = parameterBox.getSelectedItem().toString();
        File profile = null;
        logger.log("GUI: assignModificationButton: assigning profile " + selectedProfile + " to channel " + selectedTrack + " parameter " + selectedParameter);
        for(File f : profiles) {
          if (f.getName().equals(selectedProfile))
            profile = f;
        }
        if (selectedTrack.equals("All")) { //if assigning to all tracks
          if (selectedParameter.toString().equals("All")) { //if assigning to all parameters as well
            if (!selectedProfile.equals("none")) { //if not removing assignments 
              for (Integer i = 1; i <= 16; i++) { //add assignments to all tracks and parameters
                modifications.put(i.toString() + "V", profile);
                modifications.put(i.toString() + "T", profile);
                modifications.put(i.toString() + "A", profile);
              }
            } else modifications.clear(); //removing all assignments if selected profile == none
          } else { //if assigning to all tracks, but not to all parameters
            for (Integer i = 1; i <= 16; i++) {
              if (selectedProfile.equals("none")) //clear modifications
                modifications.remove(i.toString() + selectedParameter.substring(0,1));
              else //add assignments to all tracks
                modifications.put(i.toString() + selectedParameter.substring(0,1), profile);
            }
          }
        } else if (selectedParameter.equals("All")) { //if assigning to all parameters for a single track
          if (!selectedProfile.equals("none")) { //if not removing assignments, assign
            modifications.put(selectedTrack.substring(8) + "V", profile);
            modifications.put(selectedTrack.substring(8) + "T", profile);
            modifications.put(selectedTrack.substring(8) + "A", profile);
          } else { //remove assignments
            modifications.remove(selectedTrack.substring(8) + "V");
            modifications.remove(selectedTrack.substring(8) + "T");
            modifications.remove(selectedTrack.substring(8) + "A");
          }
        } else { //if working within a single parameter and single track
          String key = selectedTrack.substring(8) + selectedParameter.substring(0,1);
          if (selectedProfile.equals("none")) modifications.remove(key); //remove if none selected
          else modifications.put(key, profile); //add assignment
        }
        Humanizer.setMidiFile(selectedFile);
        Humanizer.setProfiles(modifications);
        //build table of selected modifications
        newlabelcontents.append("<html>\n<table cellpadding =\"2\" cellspacing =\"2\">\n");
        if (modifications.isEmpty()) {
          newlabelcontents.append("</table><br><center><b>no modifications selected\n");
	        next.setEnabled(false);
        } else {
	        next.setEnabled(true);
          for (Object o : modifications.keySet()) {
	          String key = o.toString();
	          if (key.length() == 2)
              newlabelcontents.append("<tr><td width = \"30\"><center>" + key.substring(0,1) + "</td>");
	          else newlabelcontents.append("<tr><td width = \"30\"><center>" + key.substring(0,2) + "</td>");
          	if (key.endsWith("V")) newlabelcontents.append("<td width = \"100\"><center>Velocity</td>"); 
          	if (key.endsWith("T")) newlabelcontents.append("<td width = \"100\"><center>Timing</td>");
          	if (key.endsWith("A")) newlabelcontents.append("<td width = \"100\"><center>Amplitude </td>");
          	newlabelcontents.append("<td width = \"150\"><center>" + modifications.get(key).getName() + "</td></tr>\n");
          }
        }
      	newlabelcontents.append("</html>");
      	modificationsSelected.setText(newlabelcontents.toString());
     }
    });
    
    clearModificationsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        logger.log("GUI: clearModificationsButton: clearing assigned modifications");
        modifications.clear();
        modificationsSelected.setText("<html><center><b> no modifications selected\n</html>");
        next.setEnabled(false);
      }
    });

		 /***************************************************************************
     * Interlude card (index 4)
     **************************************************************************/
		
		final JPanel card4 = new JPanel(new FlowLayout(FlowLayout.CENTER,0,200));
		humanizingBar.setString("Humanizing...");
		humanizingBar.setPreferredSize(new Dimension(400,22));
		humanizingBar.setStringPainted(true);
		humanizingBar.setAlignmentY(SwingConstants.CENTER);		
		card4.add(humanizingBar);

		




     /***************************************************************************
     * Preview card (index 5)
     **************************************************************************/

   //load images

    //instantiate and define settings for components
    final JPanel card5 = new JPanel(new BorderLayout());
    final JPanel card5top = new JPanel(new FlowLayout(FlowLayout.CENTER));
    final JPanel card5center = new JPanel(new FlowLayout(FlowLayout.CENTER,200,50));
    final JLabel card5label = new JLabel("<html><h2>Preview File Before Saving</h2></html>");
    final JButton endPreviewButton = new JButton(new ImageIcon(image4));
    final JButton endStopButton = new JButton(new ImageIcon(image5));
        time2.updateUI();
		time2.setPreferredSize(new Dimension(200,22));
		time2.setString("0:00");

    endStopButton.setVisible(false);

    //assemble components
    card5.add(card5top,BorderLayout.NORTH);
    card5.add(card5center,BorderLayout.CENTER);
    card5top.add(card5label);
    card5center.add(endPreviewButton);
    card5center.add(endStopButton);
		card5center.add(time2);
 
    //add listeners
    endPreviewButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        logger.log("GUI: endPreviewButton: previewing humanized file");
        if (!midifilehandler.loadMidiFile(selectedFile)) return; // load file, break on failure
        else {
					final String length = humanizer.getMidiSequence().getMicrosecondLength()/60000000 +
						":" + humanizer.getMidiSequence().getMicrosecondLength() % 60000000 / 1000000;
          endPreviewButton.setVisible(false);
          endStopButton.setVisible(true);
          humanizer.previewStart(); // play preview
          new Thread(new Runnable() {
            public void run() {
							sequencer = humanizer.getMidiSequencer();
              try {
                while (!humanizer.isPlaying()) {
                  Thread.sleep(500);
                }
                while (humanizer.isPlaying()) {
                  Thread.sleep(100);
									long minutes = sequencer.getMicrosecondPosition()/60000000;
            	    long seconds = sequencer.getMicrosecondPosition() % 60000000 / 1000000;
            	    String s;
            	    if (seconds < 10) s = minutes + ":" + 0 + seconds + "/" + length;
            	    else s = minutes + ":" + seconds + "/" + length;
            	    time2.setString(s);
            	    time2.setValue((int)humanizer.getMidiSequencer().getMicrosecondPosition());
                }
           		} catch (InterruptedException ie) {logger.log(ie);}
              endStopButton.setVisible(false);
              endPreviewButton.setVisible(true);
							time2.setValue(0);
							time2.setString("0:00/" + sequencer.getMicrosecondLength()/60000000 +
								":" + sequencer.getMicrosecondLength() % 60000000/ 1000000);

            }
          }).start();
        }
      }
    });

    endStopButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        logger.log("GUI: endStopButton: stopping humanized file preview");
        // stop the loaded file
        humanizer.previewStop();
				new Thread(new Runnable() {
					public void run() {
						while (time.getValue() > 0) {
							time2.setValue(0);
							time2.setString("0:00/" + sequencer.getMicrosecondLength()/60000000 +
								":" + sequencer.getMicrosecondLength() % 60000000/ 1000000);
							try {
								Thread.sleep(100);
							} catch (InterruptedException ie) {logger.log(ie);}
						}
					}
				}).start();
        endStopButton.setVisible(false);
        endPreviewButton.setVisible(true);
      }
    });
 
    /***************************************************************************
     * Finalize/save card (index 5)
     **************************************************************************/

    //load images
    
    //instantiate and define settings for components
    final JPanel card6 = new JPanel();
    final JLabel card6label = new JLabel("<html><h1>Save the Modified MIDI Sequence<h1><html>");
    final JButton saveButton = new JButton("Save As...");

    //assemble components
    card6.add(card6label);
    card6.add(fileName);
    card6.add(saveButton);
    
    //add listeners
    saveButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        logger.log("GUI: saveButton: saving humanized file");
        File newFile = new File(selectedFile,fileName.getText()); //temp patch, must perform operations on file
        fc.setSelectedFile(newFile);
        int retval = JFileChooser.ERROR_OPTION;
        try{ //show file selection window
          //fc.showSaveDialog(card5);
          retval = fc.showSaveDialog(card6);
        } catch (Exception ex) {logger.log(ex);}
        //if (fc.getSelectedFile() != null) // will never catch
        if(retval == JFileChooser.APPROVE_OPTION) { // only save if approved
          humanizer.saveFile(fc.getSelectedFile().toString());			
					JOptionPane.showMessageDialog(createProfileButton,"New MIDI Saved.");
				}
      }
    });

     /***************************************************************************
     * cards (help pages) index = prevpage + 10
     ***************************************************************************
    * Welcome help card (index 10)
     **************************************************************************/
    
    //load images

    //instantiate and define settings for components
    JPanel card10 = new JPanel();
    JLabel card10label = new JLabel("<html><h1>About MIDI Humanizer<h1>" +
    "<p>Welcome to MIDI Humanizer, brought to you by Team La-a!" +
    "<p>MIDI Humanizer is a production by:" +
    "<ul><li>Jamie White</li><li>Jason Shaffner</li><li>Jacob Peck</li></ul>" +
    "</html>");

    //assemble components
    card10.add(card10label);
    
    //add listeners

    /***************************************************************************
     * File Selection help card (index 11)
     **************************************************************************/
    
    //load images
    
    //instantiate and define settings for components
    JPanel card11 = new JPanel();
    JLabel card11label = new JLabel("<html><h1>File Selection<h1>" +
    "<p>To get started, load in a MIDI file to work on.<p>MIDI files typically" +
    "have a file extension of .mid or .midi.<p>After you've selected a MIDI " +
    "sequence to work<br>on, you may preview it with the play button, and stop the " + 
    "preview with the stop button.<p>When you are satisfied, you may continue by " +
    "clicking the next--> button.</html>");

    //assemble components
    card11.add(card11label);

    //add listeners
    
    /***************************************************************************
     * Profile Selection card (index 12)
     **************************************************************************/
    
    //load images

    //instantiate and define settings for components
    JPanel card12 = new JPanel();
    JLabel card12label = new JLabel("<html><h1>Profile Selection<h1>" +
    "<p>On this card, you may load in multiple profiles from which<br>to " +
    "apply variance to various parameters of the MIDI Sequence<br>loaded in " +
    "the previous step.<p>" + 
    "To add a profile, click the add button.  Profile files typically<br>have " +
    "a file extension of .mhp or .csv.<p>" + 
    "When you have loaded all the profiles you wish to use, click next--></html>");

   //assemble components
    card12.add(card12label);
    
    //add listeners

    /***************************************************************************
     * Parameter Selection help card (index 13)
     **************************************************************************/
    
    //load images

    //instantiate and define settings for components
    JPanel card13 = new JPanel();
    JLabel card13label = new JLabel("<html><h1>Modifications Selection<h1>" +
    "<p>On this card, you may assign the custom profiles you loaded in <br>" +
    "or the built in profiles to various parameters of the MIDI Sequence<br><p>" + 
    "To assign a parameter modification, click the assign button.<br>If the profile " +
    "selected is \"None,\" the modifications to <br>whichever parameters are " +
    "selected will be removed.<br>Press the clear button to unassign all modifications.<p>" +
    "When you have assigned all the modifications you wish to use, click next--></html>");

    //assemble components
    card13.add(card13label);

    //add listeners
    
    /***************************************************************************
     * Preview help card (index 14)
     **************************************************************************/
    
    //load images
    
    //instantiate and define settings for components
    JPanel card15 = new JPanel();
    JLabel card15label = new JLabel("<html><h1>Preview Before Saving<h1>" +
    "<p>Really? You need help for this?<br>Press the play button to preview " +
    "the MIDI Sequence<br>with the modifications you selected in the previous " +
    "step.<br>Press the stop button to stop playing the sequence.<br>" +
    "Go back to change which modifications to apply<br>by pressing the " +
    "previous button,<br>or continue by pressing \"Humanize!\"</html>");
    
    //assemble components
    card15.add(card15label);
    
    //add listeners

    /***************************************************************************
     * Finalize/Save help card (index 15)
     **************************************************************************/
    
    //load images

    //instantiate and define settings for components
    JPanel card16 = new JPanel();
    JLabel card16label = new JLabel("<html><h1>Save the Modified MIDI Sequence<h1><p>" + 
    "Press the save button to choose a directory and<br>filename to save the modified " +
    "MIDI sequence");
    
    //assemble components
    card16.add(card16label);
    
    //add listeners
    

    // assemble all the cards into the panel
    cardpanel = new JPanel(new CardLayout());
    cardpanel.add(card0,"0");
    cardpanel.add(card1,"1");
    cardpanel.add(card2,"2");
    cardpanel.add(card3,"3");
    cardpanel.add(card4,"4");
    cardpanel.add(card5,"5");
		cardpanel.add(card6,"6");
    cardpanel.add(card10,"10");
    cardpanel.add(card11,"11");
    cardpanel.add(card12,"12");
    cardpanel.add(card13,"13");
    cardpanel.add(card15,"15");
    cardpanel.add(card16,"16");
    
    
    /***************************************************************************
     * Control Panel Code (bottom button panel)
     **************************************************************************/
    
    //load images
    try {
      image2 = ImageIO.read(new File("MIDI2.png"));
      image3 = ImageIO.read(new File("help.png"));
    } catch (IOException io) {logger.log(io);}
    
    //instantiate and define settings for components
    JPanel controlPanel = new JPanel(new BorderLayout());
    JPanel controls = new JPanel(new FlowLayout(FlowLayout.CENTER));
    next = new JButton("Next-->");
    prev = new JButton("<--Previous");
    bottom = new JLabel(new ImageIcon(image2));
    help = new JButton(new ImageIcon(image3));
    
    //assemble components
    controlPanel.add(help,BorderLayout.WEST);
    controls.add(prev);
    controls.add(next);
    controlPanel.add(controls,BorderLayout.CENTER);
    controlPanel.add(bottom,BorderLayout.EAST);

    // add listeners for controls
    next.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        CardLayout cl = (CardLayout)(cardpanel.getLayout());
        currentPaneIndex++;
        if (currentPaneIndex > lastPaneIndex) currentPaneIndex = lastPaneIndex;
        currentPane = Integer.toString(currentPaneIndex);
        cl.show(cardpanel, currentPane);
        initializeCard(currentPaneIndex);
      }
    });
    
    prev.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        CardLayout cl = (CardLayout)(cardpanel.getLayout());
        if (currentPaneIndex <= lastPaneIndex) {
          currentPaneIndex--;
					if (currentPaneIndex == 4)
						currentPaneIndex = 3;
				} else
          currentPaneIndex -= 10;
        if (currentPaneIndex < 0) currentPaneIndex = 0;
        currentPane = Integer.toString(currentPaneIndex);
        cl.show(cardpanel, currentPane);
        initializeCard(currentPaneIndex);
      }
    });

    help.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        CardLayout cl = (CardLayout)(cardpanel.getLayout());
        currentPaneIndex += 10;
        if (currentPaneIndex >= lastPaneIndex+10) currentPaneIndex = lastPaneIndex+10;
        currentPane = Integer.toString(currentPaneIndex);
        cl.show(cardpanel, currentPane);
        initializeCard(currentPaneIndex);
      }
    });
    
    /***************************************************************************
     * GUI Initialization Code
     * Perform any tasks that need to be done before the GUI is shown here
     **************************************************************************/
    
    // build the gui, finally
    frame.getContentPane().add(cardpanel, BorderLayout.CENTER);
    frame.getContentPane().add(controlPanel, BorderLayout.SOUTH);
    
    // start on correct card
    CardLayout cl = (CardLayout)(cardpanel.getLayout());
    cl.show(cardpanel,currentPane);
    initializeCard(currentPaneIndex);
    
    // display the frame
    //frame.pack();
    frame.setVisible(true);
    frame.setResizable(false);
    frame.setLocationRelativeTo(null); // center on screen
  }
  
  /*****************************************************************************
   * initializeCard() Method
   * this is called whenever a new card is placed on the screen
   * be careful with this, and don't touch the breaks
   ****************************************************************************/

  static public void initializeCard(int index) {
    logger.log("GUI: initializeCard(index): initializing card " + index);
    switch(index) {
      case 0:
        next.setText("Let's Get Started");
        prev.setText("<--Previous");
        next.setEnabled(true);
        next.setVisible(true);
        prev.setEnabled(false);
        prev.setVisible(false);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(false);
        midifilehandler.stop();
        break;
      case 1:
        next.setText("Next-->");
        prev.setText("<--Previous");
        if (selectedFile == null)
          next.setEnabled(false);
        next.setVisible(true);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(true);
        midifilehandler.stop();
        break;
      case 2 :
        next.setText("Next-->");
        prev.setText("<--Previous");
        next.setEnabled(true);
        next.setVisible(true);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(true);
        midifilehandler.stop();
    	break;
     case 3 :	
        next.setText("Next-->");
        prev.setText("<--Previous");
        if (modifications.isEmpty())
	        next.setEnabled(false);
        next.setVisible(true);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(true);
        hasBeenHumanized = false;
				midifilehandler.stop();
	    	humanizer.previewStop();
        break;
		 case 4:
				next.setVisible(false);
				prev.setVisible(false);
				help.setVisible(false);
				humanizer.go();
				while (humanizer.getPasscount() == 0) {
					try {
						Thread.sleep(50);
					} catch (InterruptedException ie) {}
				}
                humanizingBar.updateUI();
				humanizingBar.setMaximum(humanizer.getPasscount());
				humanizingBar.setValue(0);
				new Thread (new Runnable() {
					public void run() {
					
					try {
						Thread.sleep(100);
						while (humanizingBar.getValue() < humanizingBar.getMaximum()) {
							humanizingBar.setValue(humanizer.getPassesCompleted());
              humanizingBar.setString("Humanizing... pass " + humanizingBar.getValue() 
                                     + " of " + humanizingBar.getMaximum());
							Thread.sleep(100);
						}
					} catch (InterruptedException ie) {}
					next.doClick();
					}
				}).start();
				break;
     case 5:
        next.setText("Humanize!");
        prev.setText("<--Previous");
        next.setEnabled(true);
        next.setVisible(true);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(true);
        midifilehandler.stop();
				new Thread(new Runnable() {
          public void run() {
            //humanizer.go();
						long minutes = humanizer.getMidiSequence().getMicrosecondLength()/60000000;
            long seconds = humanizer.getMidiSequence().getMicrosecondLength() % 60000000 / 1000000;
            String s;
            if (seconds < 10) s = "0:00/" + minutes + ":" + 0 + seconds;
            else s = "0:00/" + minutes + ":" + seconds;
            time2.setString(s);
						time2.setMaximum((int)humanizer.getMidiSequence().getMicrosecondLength());
						time2.setStringPainted(true);
          }
        }).start();
				humanizer.previewStop();
				break;
      case 6:
        prev.setText("<--Previous");
        next.setEnabled(false);
        next.setVisible(false);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(true);
        help.setVisible(true);
        bottom.setVisible(true);
        hasBeenHumanized = true;
				fileName.setText(selectedFile.getName().substring(0,selectedFile.getName().lastIndexOf(".")) + "-new.mid");
        midifilehandler.stop();
        humanizer.previewStop();
	break;
      case 10: case 11: case 12: case 13: case 14: case 15:
        prev.setText("Close Help");
        next.setEnabled(false);
        next.setVisible(false);
        prev.setEnabled(true);
        prev.setVisible(true);
        help.setEnabled(false);
        help.setVisible(false);
        bottom.setVisible(true);
        break;
      default:
        // shouldn't get here
        System.out.println("How did you end up here?  Index: " + index);
        bottom.setVisible(true);
        break;
    }
  }
  
  /*****************************************************************************
   * main() Method - Don't touch this
   ****************************************************************************/
  static public void main(String[] args) {
    logger.log("GUI: main(args): setting look and feel");
    // Use the system's Look and Feel
    try {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    } catch (Exception ex) {
      logger.log(ex);
    }
    
    logger.log("GUI: main(args): creating and showing GUI");    
    javax.swing.SwingUtilities.invokeLater(new Runnable() {
      public void run() {
        createAndShowGUI();
      }
    });
  }
}
