package ladasha.io;

import java.util.*;
import java.io.*;

public class ProfileHandler {

	private static File profile;
	private static HashMap<Double,Double> map = new HashMap<Double,Double>();
	private static ProfileHandler instance = null;
  private static Scanner sc;
  private static Logger logger = Logger.getInstance();
	
	private ProfileHandler() {}
	
	public static ProfileHandler getInstance() {
    logger.log("obtaining instance of ProfileHandler");
		if (instance == null) instance = new ProfileHandler();
		return instance;
	}

	public static void setProfile(File newProfile) {
		logger.log("ProfileHandler: setProfile(newProfile): setting profile");
    if (newProfile == null) return;
		profile = newProfile;
	}

	public static void reader() throws IOException {
		logger.log("ProfileHandler: reader(): reading profile");
    int count = 0;
		StringBuilder builder = new StringBuilder(count);
		try {
		  sc = new Scanner(new FileInputStream(profile));
		} catch (FileNotFoundException f) {logger.log(f);}
		sc.useDelimiter(",");
		try {
			while (sc.hasNext()) {
				count++;
				builder.append(sc.nextDouble() + "\n");
			}
		} catch (Exception exception) { logger.log(exception);
		} finally { sc.close(); }
		System.out.print(builder);
		System.out.print(count);
	}

	public static boolean isGaussian() {
	  logger.log("ProfileHandler: isGaussian(): checking if gaussian");
    try {
			sc = new Scanner(new FileInputStream(profile));
		} catch (FileNotFoundException f) { logger.log(f);}
		String s = sc.nextLine();	
		sc.close();
    return s.contains("[");    
	}

	public static boolean isDiscrete() { 
		logger.log("ProfileHandler: isDiscrete(): checking if discrete");
    try {
			sc = new Scanner(new FileInputStream(profile));
		} catch (FileNotFoundException f) {logger.log(f);}
		String s = sc.nextLine();
    sc.close();
		return !s.contains("[");
	}
	
	public static HashMap<Double,Double> getDiscrete() { 
		logger.log("ProfileHandler: getDiscrete(): getting discrete distribution");
    if (!isDiscrete()) return null;
		// variables
		int count = 0;
		double key = 0;
		double value = 0;
		try {
		  sc = new Scanner(new FileInputStream(profile));
		} catch (FileNotFoundException f) { logger.log(f); }
		sc.useDelimiter(",");
		// if count is divisible by 2
		while (sc.hasNext()) {
			double token = sc.nextDouble();
			if (count % 2 != 0) { // value, not a key
				map.put(key, token);
			} else key = token;
			count++;
		}
	 	for (Double d : map.keySet()) {
		  value += map.get(d);
	    map.put(d,value);
		}	  
    sc.close();
    if (0.999999 > value || value > 1.000001) return null; // ugly, incorrect, but better.
		else return map;
	}		
	
	public static Double[] getGaussian() { 
		logger.log("ProfileHandler: getGaussian(): getting gaussian distribution");
        if (!isGaussian()) return null;
		try {
		  sc = new Scanner(new FileInputStream(profile));
		} catch (FileNotFoundException f) { logger.log(f);}
    sc.useDelimiter(",");
		String mu = sc.next();
    String sigma = sc.next();
    if (mu == null || sigma == null) return null;
		Double[] meanAndVariance = new Double[2];
		meanAndVariance[0] = Double.parseDouble(mu.substring(mu.lastIndexOf("[")+1));
		meanAndVariance[1] = Double.parseDouble(sigma.substring(0,sigma.length()-1));
		sc.close();
    return meanAndVariance;
	}
}

/*
class ProfileHandlerTester {

	public static void main (String[] args) {
		ProfileHandler p = ProfileHandler.getInstance();
		ProfileHandler.setProfile(new File("~/humanizer-current/classes/StdGaus1.mhp"));
		ProfileHandler.getGaussian();
	}
}
*/
